(ns com.aurellem.run.bootstrap-0
  (:use (com.aurellem.gb gb-driver vbm))
  (:use (com.aurellem.run title save-corruption))
  (:use (com.aurellem.exp item-bridge)))


(defn-memo boot-root []
  [ [] (root)])
  
(defn-memo to-rival-name
  ([] (to-rival-name (boot-root)))
  ([script]
     (-> script
         title
         oak
         name-entry-rlm
         scroll-text
         scroll-text
         scroll-text
         scroll-text
         scroll-text
         )))

(defn-memo name-rival-bootstrap
  ([] (name-rival-bootstrap (to-rival-name)))
  ([script]
     (->> script
         (advance [] [:a])
         (advance [] [:r] DE)
         (play-moves
          [[]
           [:r] [] [:r] [] [:r] [] [:r] []
           [:r] [] [:r] [] [:r] [] [:d] []
           [:d] [:a]                         ;; space
           [:l] [] [:d] [:a]                 ;; [PK]
           [:u] [] [:u] [] [:u] [] [:l] [:a] ;; G
           [:d] [] [:d] [] [:d] [] [:r] [:a] ;; [PK]
           [:u] [] [:u] [] [:u] [] [:l] [:a] ;; G
           [:d] [] [:d] [] [:d] [] [:r] [:a] ;; [PK]

           [:d] [] [:r] [:a]                 ;; finish
           ]))))

(defn walk
  "Move the character along the given directions."
  [directions script]
  (reduce (fn [script direction]
            (move direction script))
          script directions))

(def ↑ [:u])
(def ↓ [:d])
(def ← [:l])
(def → [:r])

(defn-memo leave-house
  ([] (leave-house (name-rival-bootstrap)))
  ([script]
     (->> script
          finish-title
          start-walking
          walk-to-stairs
          walk-to-door
          (walk [↓ ↓]))))

(defn-memo to-pallet-town-edge
  ([] (to-pallet-town-edge (leave-house)))
  ([script]
     (->> script
          start-walking
          (walk [→ → → → →
                 ↑ ↑ ↑ ↑ ↑ ↑]))))

(defn end-text [script]
  (->> script
       (scroll-text)
       (play-moves [[] [:a]])))

(defn-memo start-pikachu-battle
  ([] (start-pikachu-battle
       (to-pallet-town-edge)))
  ([script]
     (->> script
          (advance [:b] [:b :a] DE)
          (scroll-text)
          (play-moves [[:b]])
          (scroll-text)
          (end-text) ;; battle begins
          (scroll-text))))

(defn-memo capture-pikachu
  ([] (capture-pikachu (start-pikachu-battle)))
  ([script]
     (->> script
          (scroll-text 2)
          (end-text))))

(defn-memo go-to-lab
  ([] (go-to-lab (capture-pikachu)))
  ([script]
     (->> script
          (scroll-text 5)
          (end-text)
          (scroll-text)
          (end-text)
          (scroll-text 8)
          (end-text)
          (scroll-text)
          (end-text))))

(defn-memo obtain-pikachu
  ([] (obtain-pikachu (go-to-lab)))
  ([script]
     (->> script
          (scroll-text)
          (play-moves
           (concat 
            (repeat 51 [])
            [[:a] []]))
          (walk [↓ ↓ → → ↑])
          (play-moves
           (concat [[] [:a]]
                   (repeat 100 [])))
          (scroll-text 9)
          (end-text)
          (scroll-text 7)

          (play-moves
           (concat
            (repeat 42 [])
                   [[:b]])))))

(defn-memo begin-battle-with-rival
  ([] (begin-battle-with-rival
        (obtain-pikachu)))
  ([script]
     (->> script
          (play-moves
           (repeat 200 [:b]))
          (play-moves
           (repeat 200 []))
          (walk [↓ ↓ ↓]))))
  
          
      



